'''
   \class ArrowWindow

    \brief The background of this window and a contained arrow changes
    based on relevant context properties.

    Contains an arrow that always points to the top edge of the
    screen.  The orientation is based on the context property
    Screen.TopEdge and the background of the window is black if
    Environment.IsDark is true and white if false.  If the
    Screen.TopEdge property's value is unknown then a big dot is
    displayed and in the case of Enviroment.IsDark the unknown value
    is reflected by gray background.
'''

from PySide.QtCore import *
from PySide.QtGui import *
from Context.Provider import *
from Context.Subscriber import *

from SvgRotateWidget import *
from QSingleItemSquareLayout import *

from qrc_arrows import *

class ArrowWindow(QMainWindow):

    def __init__(self, parent = None):
        super(ArrowWindow, self).__init__(parent)

        self.cpIsDark = ContextProperty("Environment.IsDark", self)
        self.cpEdgeUp =  ContextProperty("Screen.TopEdge", self)
        self.cpIsDark.valueChanged.connect(self.onIsDarkChanged)
        self.cpEdgeUp.valueChanged.connect(self.onEdgeUpChanged)
        self.centralWidget = QWidget(self)
        # There is a problem on QtGui that causes a segfault when using QLayout
        # This problem is still being investigated and by now only light sensor
        # is used in this example
        #self.mainLayout = QSingleItemSquareLayout(self.centralWidget)
        self.arrow = SvgRotateWidget(None, self)
        #self.mainLayout.addWidget(self.arrow)
        self.setCentralWidget(self.centralWidget)

        # set up initial state of the widgets
        self.onEdgeUpChanged()
        self.onIsDarkChanged()

    def onIsDarkChanged(self):
        p = self.palette()
        if self.cpIsDark.value() == None:
            p.setColor(QPalette.Background, Qt.gray)
        elif self.cpIsDark.value():
            p.setColor(QPalette.Background, Qt.black)
        else:
            p.setColor(QPalette.Background, Qt.white)
        self.setPalette(p)

    def onEdgeUpChanged(self):
        topEdge = self.cpEdgeUp.value()
        rotInt = 0
        if (topEdge == "top"):
            rotInt = SvgRotateWidget.UP
        elif (topEdge == "left"):
            rotInt = SvgRotateWidget.LEFT
        elif (topEdge == "bottom"):
            rotInt = SvgRotateWidget.DOWN
        elif (topEdge == "right"):
            rotInt = SvgRotateWidget.RIGHT
        else:
            self.arrow.load(":resources/dot.svg")
            return

        self.arrow.load(":resources/up.svg")
        if (rotInt):
            self.arrow.setRotation(rotInt)
