from PySide.QtCore import *
from PySide.QtGui import *

class QSingleItemSquareLayout(QLayout):

    def __init__(self, parent = None):
        super(QSingleItemSquareLayout, self).__init__(parent)
        self.item = 0
        self.lastReceivedRect = QRect(0, 0, 0, 0)
        self._geometry = QRect(0, 0, 0, 0)

    def __del__(self):
        del self.item
        del self.lastReceivedRect
        del self._geometry

    # Adds item if place isn't already taken.
    def add(self, item):
        if not self.hasItem():
            self.replaceItem(item)

    # Adds item if place isn't already taken.
    def addItem(self, item):
        if not hasItem():
            self.replaceItem(item)

    # Adds widget if place isn't already taken.
    def addWidget(self, widget):
        if not self.hasItem():
            self.replaceItem(QWidgetItem(widget))

    # Returns the item pointer and dereferences it here.
    def take(self):
        newItem = 0
        if self.hasItem():
            newItem = self.item
            self.item = 0
        return newItem

    # Returns the item pointer and dereferences it here. */
    def takeAt(self, index):
        if index != 0:
            return 0
        return self.take()

    # Returns the item pointer.
    def itemAt(self, index):
        if index != 0:
            return None
        if self.hasItem():
            return self.item
        return None

    # Checks if we have an item.
    def hasItem(self):
        return self.item != 0

    # Returns the count of items which can be either 0 or 1.
    def count(self):
        returnValue = 0
        if self.hasItem():
            returnValue = 1
        return returnValue

    # Replaces the item with the new and returns the old.
    def replaceItem(self, newItem):
        old = 0
        if self.hasItem():
            old = self.item
        self.item = newItem
        self.setGeometry(self._geometry)
        return old

    # Tells which way layout expands.
    def expandingDirections(self):
        return Qt.Horizontal | Qt.Vertical

    # Tells which size is preferred.
    def sizeHint(self):
        return self.item.minimumSize()

    # Tells minimum size.
    def minimumSize(self):
        return self.item.minimumSize()

    # Tells if heightForWidth calculations is handled.
    # It isn't since width isn't enough to calculate
    # proper size.
    def hasHeightForWidth(self):
        return False

    # Replaces lastReceivedRect.
    def setLastReceivedRect(self, rect):
        oldRect = self.lastReceivedRect
        self.lastReceivedRect = QRect(rect.topLeft(), rect.size())
        del oldRect

    # Returns geometry
    def geometry(self):
        return QRect(self._geometry)

    # Sets geometry to given size.
    def setGeometry(self, rect):
        # We check if the item is set and
        # if size is the same previously received.
        # If either is false nothing is done.
        if not self.hasItem() or self.areRectsEqual(self.lastReceivedRect, rect):
            return
        # Replace the last received rectangle.
        self.setLastReceivedRect(rect)
        # Calculate proper size for the item relative to the received size.
        properSize = self.calculateProperSize(rect.size())
        # Calculate center location in the rect and with item size.
        properLocation = self.calculateCenterLocation(rect.size(), properSize)
        # Set items geometry
        self.item.setGeometry(QRect(properLocation, properSize))
        oldRect = self._geometry
        # Cache the calculated geometry.
        self._geometry = QRect(properLocation, properSize)
        del oldRect
        # Super classes setGeometry
        super(QSingleItemSquareLayout, self).setGeometry(rect)

    # Takes the shortest side and creates QSize
    # with the shortest side as width and height. */
    def calculateProperSize(self, from_):
        properSize = QSize()
        if from_.height() < from_.width():
            properSize.setHeight(from_.height() - self.margin())
            properSize.setWidth(from_.height() - self.margin())
        else:
            properSize.setWidth(from_.width() - self.margin())
            properSize.setHeight(from_.width() - self.margin())
        return properSize

    # Calculates center location from the given height and width for item size.
    def calculateCenterLocation(self, from_, itemSize):
        centerLocation = QPoint()
        if from_.width() - from_.width()/2 - itemSize.width()/2 > 0:
            centerLocation.setX(from_.width() - from_.width()/2 - itemSize.width()/2)
        if from_.height() - from_.height()/2 - itemSize.height()/2 > 0:
            centerLocation.setY(from_.height() - from_.height()/2 - itemSize.height()/2)
        return centerLocation

    # Compares if two QRects are equal.
    def areRectsEqual(self, a, b):
        result = False
        if a.x() == b.x() and a.y() == b.y() and a.height() == b.height() and a.width() == b.width():
            result = True
        return result
