
import unittest
import time

from PySide.QtCore import *
from Context.Provider import *
from Context.Subscriber import *

from helper import UsesQCoreApplication


class PublishSubscriberTest(UsesQCoreApplication):

    def setUp(self):
        #Acquire resources
        self.timer = QTimer()
        UsesQCoreApplication.setUp(self)

    def tearDown(self):
        #Release resources
        del self.timer
        UsesQCoreApplication.tearDown(self)

    def testProviderSubscriber(self):
        def newValue():
            self.control = 1

        def newSubscriber(key):
            self.assertEqual(key, 'Screen.TopEdge')

        self.control = 0
        myService = Service(SessionBus, 'org.freedesktop.ContextKit.Commander')
        myService.setAsDefault()
        myProperty= Property('Screen.TopEdge')
        myProperty.setValue('top')
        self.assert_(myProperty.isSet())
        self.assertEqual(myProperty.value(), 'top')

        def modifyProviderValue():
            myProperty.setValue('bottom')
            self.assertEqual(self.control, 1)
            self.app.exit(0)

        myProperty.firstSubscriberAppeared.connect(newSubscriber)

        mySubscriber = ContextProperty(myProperty.key())
        mySubscriber.valueChanged.connect(newValue)
        mySubscriber.waitForSubscription()

        self.timer.timeout.connect(modifyProviderValue)
        self.timer.start(4)

        self.app.exec_()


if __name__ == '__main__':
    unittest.main()

